#!/usr/bin/env python

import fnmatch, getopt, os, pickle, pprint, sys

# this is our filesystem
fs = {}
excludes = ["/var/lib/apt/*",
            "/var/lib/dpkg/*",
            "/var/lib/aegis/*",
            "/var/cache/dpkg/*",
            "/etc/refhashlist*",
            "/sys/kernel/security/digsig/hashlist",
            "/sys/kernel/security/validator/hashlist",
            "/root/signature.gpg",
            "/root/.gnupg/trustdb.gpg",
            "/usr/share/applications/*mimeinfo.cache*"]


def addDir(d):
    p = fs
    for x in d:
        if x in p:
            p = p[x]
            continue
        p[x] = {}
        p = p[x]

def addFile(d):
    p = fs
    for x in d[:-1]:
        p = p[x]
    #if d[-1] in p:
    #    sys.exit(1)
    #else:
    p[d[-1]] = "file"

def delFile(d):
    p = fs
    for x in d[:-1]:
        # FIXME keyerror if dir/file not in package
        # File &quot;/usr/bin/instw2pickle.py&quot;, line 44, in delFile
        # p = p[x]
        # KeyError: 'etc'
        p = p[x]

    # we get to the exception if executed command has been rm -rf
    # which returns 0 even when the file is not existing
    try:
        del(p[d[-1]])
    except:
        sys.stderr.write("Trying to remove /%s which doesn't exist on the package\n" % "/".join(d))

def isFile(d):
    p = fs
    for x in d:
        p = p[x]
    if p == "file":
        return True
    return False

def isDir(d):
    p = fs
    for x in d:
        p = p[x]
    if type(p) == dict:
        return True
    return False

def isExcluded(path):
    for p in excludes:
        if fnmatch.fnmatch(path, p):
            return True
    return False

def renameFile(d, d2):
    if isFile(d):
        addDir(d[:-1])
        delFile(d)
        addDir(d2[:-1])
        addFile(d2)
    elif isDir(d):
        p1 = fs
        p2 = fs
        for x in d:
            p1 = p1[x]
        for x in d2[:-1]:
            p2 = p2[x]
        p2[d2[-1]]=p1
        p1 = fs
        for x in d[:-1]:
            p1 = p1[x]
        del p1[d[-1]]


if __name__ == "__main__":
    try:
        opts, args = getopt.getopt(sys.argv[1:], "l:", ["load="])
    except getopt.GetoptError, err:
        print str(err)
        sys.exit(2)

    for o, a in opts:
        if o in ["-l", "--load"]:
            if os.path.exists(a):
                fs = pickle.load(file(a))

    for line in file(args[0]).readlines():
        items = line.split("\t")
        if items[0] != "-1":
            if items[1] in ["mkdir"]:
                if not isExcluded(items[2]):
                    addDir(items[2].split("/")[1:])

            if items[1] in ["open", "fopen", "fopen64"]:
                if not isExcluded(items[2]):
                    addDir(items[2].split("/")[1:-1])
                    addFile(items[2].split("/")[1:])

            if items[1] in ["unlink"]:
                if not isExcluded(items[2]):
                    delFile(items[2].split("/")[1:])

            if items[1] in ["link"]:
                if not isExcluded(items[3]):
                    addFile(items[3].split("/")[1:])

            if items[1] in ["rename"]:
                if not isExcluded(items[3]):
                    renameFile(items[2].split("/")[1:], items[3].split("/")[1:])

    pickle.dump(fs, file(args[1],"wb"))
