#!/usr/bin/env python

# Author: P@draigBrady.com
# Source: http://www.pixelbeat.org/scripts/ps_mem.py

import sys, os, string, time
try:
    # md5 module is deprecated on python 2.6
    # so try the newer hashlib first
    import hashlib
    md5_new = hashlib.md5
except ImportError:
    import md5
    md5_new = md5.new

def getMemStats(pid):
    priv=[]
    shar=[]
    for line in open("/proc/"+str(pid)+"/smaps", "rb").readlines():
        line = line.decode("ascii")
        if line.startswith("Shared"):
            shar.append(line)
        elif line.startswith("Private"):
            priv.append(line)
    shared=sum([int(line.split()[1]) for line in shar])
    private=sum([int(line.split()[1]) for line in priv])
    return (private, shared)

def getCpuTotal():
    ret = 0
    stat = file("/proc/stat").read()
    for line in stat.splitlines():
        if not line.startswith("cpu "):
            continue
        for val in line.split():
            if not val.isdigit():
                continue
            ret += int(val)
    return ret

#The following matches "du -h" output
#see also human.py
def human(num, power="Ki"):
    powers=["Ki","Mi","Gi","Ti"]
    while num >= 1000: #4 digits
        num /= 1024.0
        power=powers[powers.index(power)+1]
    return "%.1f %s" % (num,power)

#sys.stdout.write(" Private  +   Shared  =  RAM used\tProgram \n\n")
#for cmd in sort_list:
#    sys.stdout.write("%8sB + %8sB = %8sB\t%s\n" % (human(cmd[1]-shareds[cmd[0]]),
#                                      human(shareds[cmd[0]]), human(cmd[1]),
#                                      cmd_with_count(cmd[0], count[cmd[0]])))

pid = sys.argv[1]

stat = file("/proc/%s/stat" % pid).read().split()
utime_before = int(stat[13])
stime_before = int(stat[14])
time_total_before = getCpuTotal()

time.sleep(1)

stat = file("/proc/%s/stat" % pid).read().split()
utime_after = int(stat[13])
stime_after = int(stat[14])
time_total_after = getCpuTotal()

user = 100 * (utime_after - utime_before) / (time_total_after - time_total_before)
system = 100 * (stime_after - stime_before) / (time_total_after - time_total_before)
private, shared = getMemStats(pid)

print "CPU: %s%% (user), %s%% (system)" % (user, system)
print "MEM: %sB (private), %sB (shared)" % (human(private), human(shared))
